;;;; -*- mode:Scheme -*- ;;;;

;;;; REMARKS

(define (atom? x) (or (symbol? x) (number? x) (null? x)))
(define (pattern-variable? x) (and (pair? x) (eq? '? (first x))))

;;;; MATCHER

(define (match p d #!optional bindings)
  "
  Purpose:	Matching
  Arguments:	Pattern, datum, optional bindings.
  Returns:	A list of bindings, which may be (), or FAIL.
  Remarks:	Pattern variables are indicated by (? <variable name>)
		Prolog's nameless variable is indicated by (? _)
  "
  (if (default-object? bindings) (set! bindings ()))
  (cond ((and (atom? p) (atom? d))
         (match-atoms p d bindings))
        ((pattern-variable? p)
         (match-variable p d bindings))
        ((and (pair? p) (pair? d))
         (match-pieces p d bindings))
        (else 'fail)))

(define (match-atoms p d bindings)
  ;;Are the pattern and datum the same:
  (if (eq? p d)
      ;;If so, return the value of BINDINGS:
      bindings
      ;;Otherwise, return FAIL.
      'fail))

(define (match-variable p d bindings)
  (let ((binding (find-binding p bindings)))
    ;;Is the pattern variable on the list of bindings:
    (if binding 
        ;;If it is, substitute its value an try again:
        (match (extract-value binding) d bindings)      
        ;;Otherwise, add new binding:
        (add-binding p d bindings))))

(define (match-pieces p d bindings)
  (let ((result (match (first p) (first d) bindings)))
    ;;See if the FIRST parts match producing new bindings:
    (if (eq? 'fail result)
	;;If they do not match, fail.
	'fail
	;;If they do match, try the cdr parts using the resulting bindings:
	(match (cdr p) (cdr d) result))))

;;;; AUXILIARY PROCEDURES

(define (add-binding pattern-variable-expression datum bindings)
 (if (eq? '_ (second pattern-variable-expression))
     bindings
     (cons (list (second pattern-variable-expression) datum) bindings)))

(define (find-binding pattern-variable-expression binding)
  (if (not (eq? '_ (second pattern-variable-expression)))
    (assoc (second pattern-variable-expression) binding)
    #f))

(define (extract-key binding) (first binding))

(define (extract-value binding) (second binding))

