;;; Makefile rule-base

;;; Assertion types:
;;; (<file name> <file type> <creation time>)
;;; (<file name 1> includes <file name 2>)
;;; (<file name 1> links-in <file name 2>)

(define current-time 10)

(define (make-makefile-rules)
  (clear-rules)
  (remember-rules
   '(r1-step/start->check-times
        IF (step is start)
	DELETE (step is start)
	ADD (step is check-times)
	SAYING ("Checking file times"))
 
;;; check if the source file includes any files that are newer
;;; than it, if so, it is out of date.  also, make the time on the 
;;; source file that includes the newer file be the newer time,
;;; to make sure that it is newer than the object.

   '(r2-step/check-src
        IF 
	(step is check-times)
	((? 1file) source (? 1time))
	((? 2file) source (? 2time))
	((? 1file) includes (? 2file))
	AND-IF
	(> '(? 2time)  '(? 1time))
	DELETE ((? 1file) source (? 1time))
	ADD 
	((? 1file) source (? 2time))
	((? 1file) is out-of-date)
	SAYING ("Source file ~a is out of date, last compiled at ~a." (? 1file) (? 1time)))

   '(r3-step/check-times->propagate-out
        IF (step is check-times)
	DELETE (step is check-times)
	ADD (step is propagate-out)
	SAYING ("Propagating the out-of-date flags"))

;;; check if the object file links in any source files that are newer
;;; than it, if so, it is out of date.

   '(r4-step/propogate-src
        IF 
	(step is propogate-out)
	((? 1file) object (? 1time))
	((? 2file) source (? 2time))
	((? 1file) links-in (? 2file))
	AND-IF
	(> '(? 2time) '(? 1time))
	ADD ((? 1file) is out-of-date)
	SAYING ("Object file ~a is out of date, last compiled at ~a." (? 1file) (? 1time)))

   '(r6-step/propagate-out->compile
        IF (step is propagate-out)
	DELETE (step is propagate-out)
	ADD (step is compile)
	SAYING ("Start compiling"))

;;; compile the objects that are out of date.
  
   '(r7-step/compile-objects
        IF
	(step is compile)
	((? file) object (? compile-time))
	((? file) is out-of-date)
	(current-time is (? time))
	DELETE
	((? file) object (? compile-time))
	((? file) is out-of-date)
	ADD
	((? file) object (? time))
	EVALUATING
	(delete-assertion '(current-time is (? time)))
	(set! current-time (1+ current-time))
	(remember-assertion (list 'current-time 'is current-time))
	SAYING("Compiling file ~a." (? file)))

   '(r8-step/compile->link
        IF (step is compile)
	DELETE (step is compile)
	ADD (step is link)
	SAYING ("Start linking"))

;;; re-link any modules that contain an object files that are newer
;;; than the module.

   '(r9-step/link
        IF
	(step is link)
	((? 1file) module (? 1time))
	((? 2file) object (? 2time))
	((? 1file) links-in (? 2file))
	(current-time is (? time))
	AND-IF 
	(> (? 2time) (? 1time))
	DELETE 
	((? 1file) module (? 1time))
	ADD 
	((? 1file) module (? time))
	EVALUATING
	(delete-assertion '(current-time is (? time)))
	(remember-assertion `(current-time is ,(1+ (? time))))
	SAYING("Linking module ~a." (? 1file)))

   '(r10-step/link->done 
         IF (step is link)
	 DELETE (step is link)
	 ADD (step is done)
	 SAYING ("Done"))))
   
(define (make-makefile-test)
  (clear-assertions)
  (remember-assertions
   '(step is start)
   '(current-time is 10)
   '(foo source 0)
   '(foo object 1)
   '(bar source 8)
   '(zap source 0)
   '(baz source 0)
   '(baz object 3)
   '(xoom source 0)
   '(xoom object 4)
   '(obj1 module 5)
   '(obj2 module 6)
   '(foo includes zap)
   '(zap includes bar)
   '(xoom includes bar)
   '(obj1 links-in foo)
   '(obj1 links-in xoom)
   '(obj2 links-in baz)))

(define (makefile-test)
  (make-makefile-rules)
  (make-makefile-test)
  (display-assertions)
  (chain)
  (display-assertions))
