;;;; -*- mode:scheme -*- ;;;;

;;;; AUXILIARY FUNCTIONS

(define (ask object message)
  (let ((method (get-method object (getr message))))
    (if (method? method)
	(method object)
	'unknown)))

(define (tell object message value)
  (let ((method (get-method object (setr message))))
    (if (method? method)
	(begin (method object value) #t)
	#f)))

(define (getr message)
  (cond	((eq? message 'item) 'get-item)
	((eq? message 'price) 'get-price)
	((eq? message 'size) 'get-size)
	((eq? message 'color) 'get-color)
	((eq? message 'material) 'get-material)
	((eq? message 'origin) 'get-origin)
	(else message)))

(define (setr message)
  (cond	((eq? message 'item) 'set-item)
	((eq? message 'price) 'set-price)
	((eq? message 'size) 'set-size)
	((eq? message 'color) 'set-color)
	((eq? message 'material) 'set-material)
	((eq? message 'origin) 'set-origin)
	(else message)))

(define (get-method object message) (object message))

(define (method? x) x)

;;;; FRAME CONSTRUCTORS

(define (make-rug item)
  (let ((price 'unknown)
	(size 'medium)
	(color 'red)
	(material 'wool))

  (define (get-item x) item)
  (define (get-price x) price)
  (define (get-size x) size)
  (define (get-color x) color)
  (define (get-material x) material)

  (define (set-price item x) (set! price x))
  (define (set-size item x) (set! size x))
  (define (set-color item x) (set! color x))
  (define (set-material item x) (set! material x))

  (lambda (message)
    (cond ((eq? message 'get-item) get-item)
	  ((eq? message 'get-price) get-price)
	  ((eq? message 'get-size) get-size)
	  ((eq? message 'get-color) get-color)
	  ((eq? message 'get-material) get-material)

	  ((eq? message 'set-price) set-price)
	  ((eq? message 'set-size) set-size)
	  ((eq? message 'set-color) set-color)
	  ((eq? message 'set-material) set-material)

	  (else #f)))))

(define (make-turkish-rug item)
  (let ((rug (make-rug item))
	(origin 'turkey))
  (define (get-origin x) origin)
  (lambda (message)
    (cond ((eq? message 'get-origin) get-origin)
	  (else (get-method rug message))))))

(define (make-chinese-rug item)
  (let ((rug (make-rug item))
	(material 'silk)
	(color 'blue)
	(size 'small)
	(origin 'china))
  (define (get-size x) size)
  (define (get-color x) color)
  (define (get-material x) material)
  (define (get-origin x) origin)

  (define (set-size item x) (set! size x))
  (define (set-color item x) (set! color x))

  (lambda (message)
    (cond ((eq? message 'get-size) get-size)
	  ((eq? message 'get-color) get-color)
	  ((eq? message 'get-material) get-material)
	  ((eq? message 'get-origin) get-origin)

	  ((eq? message 'set-size) set-size)
	  ((eq? message 'set-color) set-color)
	  
	  (else (get-method rug message))))))


;;;; DATA

(define i-730 (make-turkish-rug 'I-730))
 (tell i-730 'price 3500)
 (tell i-730 'color 'green)
(define I-827 (make-chinese-rug 'I-827))
 (tell I-827 'price 300)
(define I-314 (make-chinese-rug 'I-314))
 (tell I-314 'price 320)
(define I-812 (make-chinese-rug 'I-812))
 (tell I-812 'price 1050)
 (tell I-812 'size 'large)
(define I-372 (make-chinese-rug 'I-372))
 (tell I-372 'price 2300)
 (tell I-372 'color 'red)
(define I-370 (make-turkish-rug 'I-370))
 (tell I-370 'price 3000)
 (tell I-370 'color 'green)
(define I-282 (make-chinese-rug 'I-182))
 (tell I-282 'price 5150)
(define I-134 (make-turkish-rug 'I-134))
 (tell I-134 'price 8320)
 (tell I-134 'size 'small)
 (tell I-134 'material 'silk)
(define I-182 (make-turkish-rug 'I-182))
 (tell I-182 'price 1700)
 (tell I-182 'size 'large)
(define I-732 (make-chinese-rug 'I-732))
 (tell I-732 'price 2300)
(define I-811 (make-turkish-rug 'I-812))
 (tell I-811 'price 1500)
 (tell I-811 'color 'green)
 (tell I-811 'material 'silk)
(define I-427 (make-turkish-rug 'I-427))
 (tell I-427 'price 2500)
(define I-737 (make-turkish-rug 'I-737))
 (tell I-737 'price 500)
 (tell I-737 'size 'small)
 (tell I-737 'material 'silk)
(define I-287 (make-chinese-rug 'I-287))
 (tell I-287 'price 5300)
(define I-247 (make-chinese-rug 'I-247))
 (tell I-247 'price 2530)
(define I-377 (make-turkish-rug 'I-377))
 (tell I-377 'price 2525)
 (tell I-377 'material 'silk)

(define rugs
  (list I-730 I-827 I-314 I-812	I-372 I-370 I-282 I-134
	I-182 I-732 I-811 I-427 I-737 I-287 I-247 I-377))


