;;;; -*- mode:scheme -*- ;;;;

;;;; ATTRIBUTES

(define attribute-list
  (define-tree
    '(branch
      ((attribute --> a) (attribute-list --> a-list) rtn (cons a a-list))
      (and (attribute --> a) rtn (list a))
      ((attribute --> a) rtn (list a)))))

(define attribute
  (define-tree
    '(branch (origin       rtn 'origin)
	     (origins      rtn 'origin)
	     (color        rtn 'color)
	     (colors       rtn 'color)		
	     (size         rtn 'size)
	     (sizes        rtn 'size)	
	     (material     rtn 'material)
	     (materials    rtn 'material)
	     (price        rtn 'price)
	     (price        rtn 'price)
	     (cost         rtn 'price)
	     (costs        rtn 'price))))

;;;; ATTRIBUTE VALUES

(define attribute-value-list
  (define-tree
    '(branch ((attribute-value --> a-v)
	      (attribute-value-list --> a-v-l)
	      rtn (append a-v a-v-l))
	     (and (attribute-value --> a-v) rtn a-v)
	     ((attribute-value --> a-v) rtn a-v))))

(define attribute-value
  (define-tree
    '(branch ((origin-value --> v) rtn v)
	     ((size-value --> v) rtn v)
	     ((color-value --> v) rtn v)
	     ((material-value --> v) rtn v)
	     ((price-value --> v) rtn v))))

(define origin-value
  (define-tree
    '(branch (turkey      rtn '(origin eq? turkey))
	     (china       rtn '(origin eq? china))
	     (turkish      rtn '(origin eq? turkey))
	     (chinese       rtn '(origin eq? china)))))

(define size-value
  (define-tree
    '(branch (large        rtn '(size eq? large))
	     (medium       rtn '(size eq? medium))
	     (small        rtn '(size eq? small)))))

(define color-value
  (define-tree
    '(branch (blue         rtn '(color eq? blue))
	     (red          rtn '(color eq? red))
	     (green        rtn '(color eq? green)))))

(define material-value
  (define-tree
    '(branch (silk        rtn '(material eq? silk))
	     (wool        rtn '(material eq? wool)))))

(define price-value-list
  (define-tree
    '(branch (that
              (price-value --> p-v)
              (price-value-list --> p-v-l)
              rtn (append p-v p-v-l))
             (that (price-value --> p-v) rtn p-v))))

(define opt-that
  (define-tree
    '(branch ()
             (which)
             (that))))

(define opt-and
  (define-tree
    '(branch ()
             (and))))

(define price-value
  (define-tree
    '(branch (cost (number-comparison --> n-c)    rtn (cons 'price n-c))
             (price (number-comparison --> n-c)   rtn (cons 'price n-c)))))

  

;;;; OBJECTS

(define object-description
  (define-tree
    '(branch ((determiner) (object-description --> o) rtn o)
             ((attribute-value-list --> a-v-l)
              (object-description --> o)
              rtn `(db-select ,o '(with ,@a-v-l)))
	     ((object --> o)
	      that cost
	      (compare-price --> p)
	      rtn `(db-select ,o '(with ,@p)))
	     ((object --> o) rtn o))))

(define compare-price
  (define-tree
    '(branch (less than (number --> n) rtn `(price < ,n))
             (more than (number --> n) rtn `(price > ,n)))))


(define determiner
  (define-tree
    '(branch (a) (the))))

(define object
  (define-tree
    '(branch (rug	rtn '(db-select rugs '()))
	     (rugs	rtn '(db-select rugs '())))))

;;;; MAIN TREE

(define db-call display)

(define main
  (define-tree
    '(branch
      (count (object-description --> o)
	if-end-rtn (db-call `(db-report-count (db-count-items ,o))))
      (how many (object-description --> o) are there
	if-end-rtn (db-call `(db-report-count (db-count-items ,o))))
      (what (attribute-list --> a-l) are (object-description --> o)
	if-end-rtn (db-call `(db-show (db-project ,o '(over ,@a-l)))))
      (what are (object-description --> o) (attribute-list --> a-l)
	if-end-rtn (db-call `(db-show (db-project ,o '(over ,@a-l)))))
      ((identify)
	 branch
	 ((object-description --> o)
	  if-end-rtn (db-call `(db-show ,o)))
	 (the (attribute-list --> a-l) of (object-description --> o)
	  if-end-rtn (db-call `(db-show (db-project ,o '(over ,@a-l))))))
      ((rank) (object-description --> o) by (attribute-list --> a-l)
       if-end-rtn (db-call `(db-show (db-sort ,o '(by ,@a-l)))))
      (stop if-end-rtn 'quit)
      (rtn (format #t "~%Sorry, I do not understand")
	   #f))))

;;;; MISCELLANEOUS

(define (number words)
  (cond ((null? words) #f)
	((number? (first words)) (list (first words) (cdr words)))
	(else #f)))

(define rank
  (define-tree
    '(branch (rank)
	     (list)
	     (sort))))

(define identify
  (define-tree
    '(branch (identify)
	     (describe)
	     (show me all)
	     (show me)
	     (what is)
	     (what are)
	     (who is)
	     (who are)
	     (give)
	     (display)
	     (print)
	     (list)
	     (present)
	     (which))))





