;;;; -*- mode:Scheme -*- ;;;;

;;;; COMMONLISP IMITATORS

(define (remove-duplicates l)
  (cond ((null? l) ())
	((member (first l) (cdr l)) (remove-duplicates (cdr l)))
	(else (cons (first l) (remove-duplicates (cdr l))))))

(define (position x l #!optional count)
  (if (default-object? count) (set! count 0))
  (cond ((null? l) #f)
	((eq? x (first l)) count)
	(else (position x (cdr l) (1+ count)))))

;;;; BASIC PROCEDURES

(define (db-add record relation)
  (if (member record (cdr relation))
      relation
    (cons (first relation) (cons record (cdr relation)))))

(define (db-delete record relation)
  (cons (first relation) (delete record (cdr relation))))

(define (db-combine #!rest relations)
  (if (not (= 1 (length (remove-duplicates (map first relations)))))
      (error "You are trying to combine incompatible data with db-combine"))
  (cons (first (first relations))
	(remove-duplicates
	  (apply append (map cdr relations)))))

(define (db-combine-distinct #!rest relations)
  (if
   (not (= 1 (length (remove-duplicates (map first relations)))))
   (error
    "You are trying to combine incompatible relations"))
  (cons (first (first relations))
	(apply append (map cdr relations))))

(define (db-project relation field-list)
  (set! relation (db-project-distinct relation field-list))
  (cons (first relation) (remove-duplicates (cdr relation))))

(define (db-project-distinct relation field-list)
  (set! field-list (delete 'over (delete 'and field-list)))
  (db-project-aux relation field-list))

(define (db-project-aux relation projections)
  (let* ((fields (first relation))
	 (positions (map (lambda (p) (position p fields)) projections)))
    (if (member #f positions)
	(error "You cannot project over a field that does not exist"))
    (map (lambda (record)
	   (map (lambda (index) (list-ref record index))
		positions))
	 relation)))

(define (db-select relation triple-list)
  (db-select-aux relation (delete 'with (delete 'and triple-list))))
  
(define (db-select-aux relation triple-list)
  (cond ((null? triple-list) relation)
        ((eq? '* (third triple-list))
         (db-select-aux relation (cdddr triple-list)))
	(else
	 (let ((var (first triple-list))
               (function (eval (second triple-list) (the-environment)))
               (reference (third triple-list)))
           (append-map
            (lambda (frame)
              (if (function (ask frame var) reference)
                  (list frame)
                  ()))
            (db-select-aux relation (cdddr triple-list)))))))

;;;; COUNTING AND SORTING PROCEDURES

(define (db-report-count n)
  (cond ((= n 0) (format #t "~%There are none"))
	((= n 1) (format #t "~%There is 1"))
	(else (format #t "~%There are ~a" n))))

(define (db-count-items relation)
  (length relation))

(define (db-sort relation keys)
  (db-sort-aux relation (delete 'by (delete 'and keys))))

(define (db-sort-aux relation keys)
  (let ((fields (first relation))
	(records (cdr relation)))
    (cond ((null? keys) relation)
	  (else
	   (let ((index (position (first keys) fields)))
	     (db-sort-aux
	      (cons fields
		    (sort records
			  (lambda (x y)
			    (less? (list-ref x index)
				   (list-ref y index)))))
	      (cdr keys)))))))

(define (less? x y)
  (cond ((and (number? x) (number? y)) (< x y))
	(else (string<? (format #f "~a" x) (format #f "~a" y)))))

;;;; RELATION DISPLAY

(define (db-show arg)
  (db-show-relation (cons '(item origin color size material price) 
			  (map get-frame-values arg))))

(define (get-frame-values item)
  `(,(ask item 'item)
    ,(ask item 'origin)
    ,(ask item 'color)
    ,(ask item 'size)
    ,(ask item 'material)
    ,(ask item 'price)))

(define (db-show-relation arg)
  (let ((widths (db-field-widths arg)))
    (format #t "~%")
    (db-show-record widths (first arg))
    (db-show-record
     widths
     (map (lambda (width) (make-string width #\-)) widths))
    (for-each (lambda (record) (db-show-record widths record))
	      (cdr arg))))

(define (db-show-record widths fields)
  (format #t "~%|")
      (do ((widths widths (cdr widths))
           (fields fields (cdr fields)))
	  ((null? widths))
	(format #t " ~a~a |"
		(first fields)
		(make-string
		  (- (first widths)
		     (string-length (format #f "~a" (first fields))))
		  #\space))))

(define (db-field-widths relation)
  (cond ((null? (cdr relation))
	 (map string-length
	      (map (lambda (x) (format #f "~a" x))
		   (first relation))))
	(else (map max
		   (db-field-widths (list (first relation)))
		   (db-field-widths (cdr relation))))))


;;;; INTERFACE EVALUATION PROCEDURE


(define (db-call X) (eval x (the-environment)))

