;;;; -*- mode:Lisp; syntax:Common-Lisp; package:user -*- ;;;;

;;;; SEMANTIC TRANSITION TREE TRANSLATOR

(define (define-tree tree)
  (eval `(lambda (word-list) ,(translate-tree tree))
	(the-environment)))

(define (translate-tree tree)
  "
  Purpose:	Translate grammar into an executable expression.
  Remarks:	Contains a COND clause for every grammar element.
		See the grammar data file for sample use.
  "
  (cond ((null? tree) '(list #t word-list))
        ((eq? 'branch (first tree))
	 (translate-branches (cdr tree)))
        ((list? (first tree))
	 (translate-subtree-call (first tree) (cdr tree)))
        ((eq? 'rtn (first tree))
	 (if (null? (cdr tree))
	     `(list #t word-list)
	     `(list (begin ,@(cdr tree)) word-list)))
        ((eq? 'if-end-rtn (first tree))
	 (if (null? (cdr tree))
	     `(if (null? word-list)
		  (list #t '())
		  #f)
	     `(if (null? word-list)
		  (list (begin ,@(cdr tree)) '())
		  #f)))
        (else `(if (null? word-list)
		   #f
		   (let ((current-word (first word-list))
			 (word-list (cdr word-list)))
		     (if (eq? current-word ',(first tree))
			 ,(translate-tree (cdr tree))
			 #f))))))

(define (translate-branches branches)
  `(or ,@(map translate-tree branches)))

(define (translate-subtree-call call-description rest-of-tree)
  (cond ((null? (cdr call-description))
	 `(let ((result (,(first call-description) word-list)))
	    (if result
		(let ((word-list (second result)))
		  ,(translate-tree rest-of-tree))
		#f)))
	((and (eq? '--> (second call-description))
	      (list? (cddr call-description)))
	 `(let ((result (,(first call-description) word-list)))
	    (if result
		(let ((,(third call-description) (first result))
		      (word-list (second result)))
		  ,(translate-tree rest-of-tree))
		#f)))
	(else (error "Unrecognized subtree syntax" call-description))))

;;;; TOP-LEVEL INTERFACE

(define (run)
  (define (loop)
    (format #t "~%~%> ")
    (let ((result (main (read-sentence))))
      (if (not (and (list? result) (eq? 'quit (first result))))
	  (loop))))
  (format #t "~%~%Welcome! Please type a question with a period.")
  (loop))

(define (read-sentence)
  (define (read-words)
    (let ((word (read)))
      (if (eof-object? word)
	  '()
	  (cons word (read-words)))))
  (let* ((terminators (char-set #\newline #\linefeed))
	 (sentence (read-string terminators)))
    (read-char)
    (set! sentence (string-trim-right sentence char-set:alphanumeric))
    (with-input-from-string sentence read-words)))


;;;; ALTERNATE TOP-LEVEL INTERFACE

(define (walk)
  (define (loop)
    (format #t "~%~%> ")
    (let ((result (main (read))))
      (if (not (and (list? result) (eq? 'quit (first result))))
	  (loop))))
  (format #t "~%~%Welcome! Please type a parenthesized question.")
  (loop))
