;;; For schedules & flights

(define empty-queue? null?)
(define first-flight first)
(define remaining-flights cdr)
(define construct-queue list)
(define construct-flight cons)



;;;; GENERALIZED SEARCH PROCEDURE

(define (generalized-search merge-flights-into-queue
			    #!optional queue)
  (if (default-object? queue)
      (set! queue (construct-queue (construct-flight '() *schedule*))))
  (cond
    ;;If the queue is empty, lose; no flight can be found:
    ((empty-queue? queue) #f)
    ;;If the first flight in the queue terminates at the finish, win:
    ((empty-queue? (remaining-flights (first-flight queue)))
     (pp (first-flight (first-flight queue)))
     (first-flight (first-flight queue)))
    ;;Otherwise, try again with a new, method-specific queue:
    (else (generalized-search merge-flights-into-queue
            (merge-flights-into-queue (extend (first-flight queue))
				    (remaining-flights queue))))))

(define (extend flight)
  ;;Produce a list of extended flights:
  (map (lambda (plane)
	 (construct-flight (add-assignment 
		plane
		(first-flight (remaining-flights flight))
		(first-flight flight))
		       (cdr (remaining-flights flight))))
       *fleet*))


(define (stupid)
  ;;Define a queue constructor specific to depth-first search;
  ;;the depth-first-specific queue constructor adds the new paths in front:
  (define (combine-paths new-paths old-paths #!rest ignore)
    (append new-paths old-paths))
  ;;Fire up generalized search using depth-first-specific queue constructor:
  (generalized-search combine-paths))


(define (remove-if predicate elements)
  (append-map (lambda (e) (if (predicate e) () (list e))) elements))

(define (remove-if-not predicate elements)
  (append-map (lambda (e) (if (predicate e) (list e) ())) elements))


(define (basic)
  (define (combine-paths new-paths old-paths #!rest ignore)
    (append new-paths old-paths))
  (pp (first-flight (remove-if-not
		     valid-fleet-assignment? 
		     (generalized-search.new combine-paths '() )))))


(define (generalized-search.new merge-flights-into-queue all #!optional queue)

  (if (default-object? queue)
      (set! queue (construct-queue (construct-flight '() *schedule*))))
  (cond
    ;;If the queue is empty, lose; no flight can be found:
    ((empty-queue? queue) #f)
    ;;If the first flight in the queue terminates at the finish, win:
    ((empty-queue? (remaining-flights (first-flight queue)))
     (cons all (first-flight (first-flight queue)))
     (first-flight (first-flight queue)))
    ;;Otherwise, try again with a new, method-specific queue:

    (else (generalized-search.new merge-flights-into-queue all
				  (construct-queue 
				   (merge-flights-into-queue 
				    (extend (first-flight queue))
				    (remaining-flights queue)) )))))










(define (generalized-search.eng merge-flights-into-queue
			    #!optional queue)
  (let ((all-paths '()))
    (define (gen-search-eng queue)
      (cond
       ;;If the queue is empty, lose; no flight can be found:
       ((empty-queue? queue) #f)
       ;;If the first flight in the queue terminates at the finish, win:
       ((empty-queue? (remaining-flights (first-flight queue)))
	(set! all-paths
	      (cons (car (first-flight queue)) all-paths))
        (gen-search-eng (remaining-flights queue)))
       ;;Otherwise, try again with a new, method-specific queue:
       (else (generalized-search.eng (merge-flights-into-queue 
				      (extend (first-flight queue))
				      (remaining-flights queue))))))
  (gen-search-eng (construct-queue (construct-flight '() *schedule*)))
  all-paths))
